@@ -12,9 +12,8 @@ static const uint8_t PM_10_0_VALUE_INDEX = 7;
 
 void HM3301Component::setup() {
   ESP_LOGCONFIG(TAG, "Setting up HM3301...");
-  hm3301_ = new HM330X();
-  error_code_ = hm3301_->init();
-  if (error_code_ != NO_ERROR) {
+  if (i2c::ERROR_OK != this->write(&SELECT_COMM_CMD, 1)) {
+    error_code_ = ERROR_COMM;
     this->mark_failed();
     return;
   }
@@ -36,7 +35,7 @@ void HM3301Component::dump_config() {
 float HM3301Component::get_setup_priority() const { return setup_priority::DATA; }
 
 void HM3301Component::update() {
-  if (!this->read_sensor_value_(data_buffer_)) {
+  if (this->read(data_buffer_, 29) != i2c::ERROR_OK) {
     ESP_LOGW(TAG, "Read result failed");
     this->status_set_warning();
     return;
@@ -85,8 +84,6 @@ void HM3301Component::update() {
   this->status_clear_warning();
 }
 
-bool HM3301Component::read_sensor_value_(uint8_t *data) { return !hm3301_->read_sensor_value(data, 29); }
-
 bool HM3301Component::validate_checksum_(const uint8_t *data) {
   uint8_t sum = 0;
   for (int i = 0; i < 28; i++) {
