@@ -141,7 +141,9 @@ def register_addressable_effect(
     },
 )
 async def lambda_effect_to_code(config, effect_id):
-    lambda_ = await cg.process_lambda(config[CONF_LAMBDA], [], return_type=cg.void)
+    lambda_ = await cg.process_lambda(
+        config[CONF_LAMBDA], [(bool, "initial_run")], return_type=cg.void
+    )
     return cg.new_Pvariable(
         effect_id, config[CONF_NAME], lambda_, config[CONF_UPDATE_INTERVAL]
     )
