@@ -4,7 +4,7 @@
 namespace esphome {
 namespace pid {
 
-static const char *TAG = "pid.climate";
+static const char *const TAG = "pid.climate";
 
 void PIDClimate::setup() {
   this->sensor_->add_on_state_callback([this](float state) {
@@ -20,7 +20,13 @@ void PIDClimate::setup() {
     restore->to_call(this).perform();
   } else {
     // restore from defaults, change_away handles those for us
-    this->mode = climate::CLIMATE_MODE_HEAT_COOL;
+    if (supports_heat_() && supports_cool_()) {
+      this->mode = climate::CLIMATE_MODE_HEAT_COOL;
+    } else if (supports_cool_()) {
+      this->mode = climate::CLIMATE_MODE_COOL;
+    } else if (supports_heat_()) {
+      this->mode = climate::CLIMATE_MODE_HEAT;
+    }
     this->target_temperature = this->default_target_temperature_;
   }
 }
@@ -30,19 +36,25 @@ void PIDClimate::control(const climate::ClimateCall &call) {
   if (call.get_target_temperature().has_value())
     this->target_temperature = *call.get_target_temperature();
 
-  // If switching to non-auto mode, set output immediately
-  if (this->mode != climate::CLIMATE_MODE_HEAT_COOL)
-    this->handle_non_auto_mode_();
+  // If switching to off mode, set output immediately
+  if (this->mode == climate::CLIMATE_MODE_OFF)
+    this->write_output_(0.0f);
 
   this->publish_state();
 }
 climate::ClimateTraits PIDClimate::traits() {
   auto traits = climate::ClimateTraits();
   traits.set_supports_current_temperature(true);
-  traits.set_supports_heat_cool_mode(true);
   traits.set_supports_two_point_target_temperature(false);
-  traits.set_supports_cool_mode(this->supports_cool_());
-  traits.set_supports_heat_mode(this->supports_heat_());
+
+  traits.set_supported_modes({climate::CLIMATE_MODE_OFF});
+  if (supports_cool_())
+    traits.add_supported_mode(climate::CLIMATE_MODE_COOL);
+  if (supports_heat_())
+    traits.add_supported_mode(climate::CLIMATE_MODE_HEAT);
+  if (supports_heat_() && supports_cool_())
+    traits.add_supported_mode(climate::CLIMATE_MODE_HEAT_COOL);
+
   traits.set_supports_action(true);
   return traits;
 }
@@ -72,14 +84,15 @@ void PIDClimate::write_output_(float value) {
 
   // Update action variable for user feedback what's happening
   climate::ClimateAction new_action;
-  if (this->supports_cool_() && value < 0)
+  if (this->supports_cool_() && value < 0) {
     new_action = climate::CLIMATE_ACTION_COOLING;
-  else if (this->supports_heat_() && value > 0)
+  } else if (this->supports_heat_() && value > 0) {
     new_action = climate::CLIMATE_ACTION_HEATING;
-  else if (this->mode == climate::CLIMATE_MODE_OFF)
+  } else if (this->mode == climate::CLIMATE_MODE_OFF) {
     new_action = climate::CLIMATE_ACTION_OFF;
-  else
+  } else {
     new_action = climate::CLIMATE_ACTION_IDLE;
+  }
 
   if (new_action != this->action) {
     this->action = new_action;
@@ -87,23 +100,9 @@ void PIDClimate::write_output_(float value) {
   }
   this->pid_computed_callback_.call();
 }
-void PIDClimate::handle_non_auto_mode_() {
-  // in non-auto mode, switch directly to appropriate action
-  //  - HEAT mode / COOL mode -> Output at ±100%
-  //  - OFF mode -> Output at 0%
-  if (this->mode == climate::CLIMATE_MODE_HEAT) {
-    this->write_output_(1.0);
-  } else if (this->mode == climate::CLIMATE_MODE_COOL) {
-    this->write_output_(-1.0);
-  } else if (this->mode == climate::CLIMATE_MODE_OFF) {
-    this->write_output_(0.0);
-  } else {
-    assert(false);
-  }
-}
 void PIDClimate::update_pid_() {
   float value;
-  if (isnan(this->current_temperature) || isnan(this->target_temperature)) {
+  if (std::isnan(this->current_temperature) || std::isnan(this->target_temperature)) {
     // if any control parameters are nan, turn off all outputs
     value = 0.0;
   } else {
@@ -128,8 +127,8 @@ void PIDClimate::update_pid_() {
     }
   }
 
-  if (this->mode != climate::CLIMATE_MODE_HEAT_COOL) {
-    this->handle_non_auto_mode_();
+  if (this->mode == climate::CLIMATE_MODE_OFF) {
+    this->write_output_(0.0);
   } else {
     this->write_output_(value);
   }
