@@ -1,3 +1,5 @@
+#ifdef USE_ARDUINO
+
 #include "prometheus_handler.h"
 #include "esphome/core/application.h"
 
@@ -5,7 +7,7 @@ namespace esphome {
 namespace prometheus {
 
 void PrometheusHandler::handleRequest(AsyncWebServerRequest *req) {
-  AsyncResponseStream *stream = req->beginResponseStream("text/plain");
+  AsyncResponseStream *stream = req->beginResponseStream("text/plain; version=0.0.4; charset=utf-8");
 
 #ifdef USE_SENSOR
   this->sensor_type_(stream);
@@ -43,6 +45,12 @@ void PrometheusHandler::handleRequest(AsyncWebServerRequest *req) {
     this->switch_row_(stream, obj);
 #endif
 
+#ifdef USE_LOCK
+  this->lock_type_(stream);
+  for (auto *obj : App.get_locks())
+    this->lock_row_(stream, obj);
+#endif
+
   req->send(stream);
 }
 
@@ -55,7 +63,7 @@ void PrometheusHandler::sensor_type_(AsyncResponseStream *stream) {
 void PrometheusHandler::sensor_row_(AsyncResponseStream *stream, sensor::Sensor *obj) {
   if (obj->is_internal())
     return;
-  if (!isnan(obj->state)) {
+  if (!std::isnan(obj->state)) {
     // We have a valid value, output this value
     stream->print(F("esphome_sensor_failed{id=\""));
     stream->print(obj->get_object_id().c_str());
@@ -125,7 +133,7 @@ void PrometheusHandler::fan_type_(AsyncResponseStream *stream) {
   stream->print(F("#TYPE esphome_fan_speed GAUGE\n"));
   stream->print(F("#TYPE esphome_fan_oscillation GAUGE\n"));
 }
-void PrometheusHandler::fan_row_(AsyncResponseStream *stream, fan::FanState *obj) {
+void PrometheusHandler::fan_row_(AsyncResponseStream *stream, fan::Fan *obj) {
   if (obj->is_internal())
     return;
   stream->print(F("esphome_fan_failed{id=\""));
@@ -249,7 +257,7 @@ void PrometheusHandler::cover_type_(AsyncResponseStream *stream) {
 void PrometheusHandler::cover_row_(AsyncResponseStream *stream, cover::Cover *obj) {
   if (obj->is_internal())
     return;
-  if (!isnan(obj->position)) {
+  if (!std::isnan(obj->position)) {
     // We have a valid value, output this value
     stream->print(F("esphome_cover_failed{id=\""));
     stream->print(obj->get_object_id().c_str());
@@ -308,5 +316,31 @@ void PrometheusHandler::switch_row_(AsyncResponseStream *stream, switch_::Switch
 }
 #endif
 
+#ifdef USE_LOCK
+void PrometheusHandler::lock_type_(AsyncResponseStream *stream) {
+  stream->print(F("#TYPE esphome_lock_value GAUGE\n"));
+  stream->print(F("#TYPE esphome_lock_failed GAUGE\n"));
+}
+void PrometheusHandler::lock_row_(AsyncResponseStream *stream, lock::Lock *obj) {
+  if (obj->is_internal())
+    return;
+  stream->print(F("esphome_lock_failed{id=\""));
+  stream->print(obj->get_object_id().c_str());
+  stream->print(F("\",name=\""));
+  stream->print(obj->get_name().c_str());
+  stream->print(F("\"} 0\n"));
+  // Data itself
+  stream->print(F("esphome_lock_value{id=\""));
+  stream->print(obj->get_object_id().c_str());
+  stream->print(F("\",name=\""));
+  stream->print(obj->get_name().c_str());
+  stream->print(F("\"} "));
+  stream->print(obj->state);
+  stream->print('\n');
+}
+#endif
+
 }  // namespace prometheus
 }  // namespace esphome
+
+#endif  // USE_ARDUINO
