@@ -1,10 +1,11 @@
 #include "shtcx.h"
 #include "esphome/core/log.h"
+#include "esphome/core/hal.h"
 
 namespace esphome {
 namespace shtcx {
 
-static const char *TAG = "shtcx";
+static const char *const TAG = "shtcx";
 
 static const uint16_t SHTCX_COMMAND_SLEEP = 0xB098;
 static const uint16_t SHTCX_COMMAND_WAKEUP = 0x3517;
@@ -25,6 +26,7 @@ inline const char *to_string(SHTCXType type) {
 
 void SHTCXComponent::setup() {
   ESP_LOGCONFIG(TAG, "Setting up SHTCx...");
+  this->wake_up();
   this->soft_reset();
 
   if (!this->write_command_(SHTCX_COMMAND_READ_ID_REGISTER)) {
@@ -110,18 +112,20 @@ uint8_t sht_crc(uint8_t data1, uint8_t data2) {
 
   crc ^= data1;
   for (bit = 8; bit > 0; --bit) {
-    if (crc & 0x80)
+    if (crc & 0x80) {
       crc = (crc << 1) ^ 0x131;
-    else
+    } else {
       crc = (crc << 1);
+    }
   }
 
   crc ^= data2;
   for (bit = 8; bit > 0; --bit) {
-    if (crc & 0x80)
+    if (crc & 0x80) {
       crc = (crc << 1) ^ 0x131;
-    else
+    } else {
       crc = (crc << 1);
+    }
   }
 
   return crc;
@@ -129,10 +133,9 @@ uint8_t sht_crc(uint8_t data1, uint8_t data2) {
 
 bool SHTCXComponent::read_data_(uint16_t *data, uint8_t len) {
   const uint8_t num_bytes = len * 3;
-  auto *buf = new uint8_t[num_bytes];
+  std::vector<uint8_t> buf(num_bytes);
 
-  if (!this->parent_->raw_receive(this->address_, buf, num_bytes)) {
-    delete[](buf);
+  if (this->read(buf.data(), num_bytes) != i2c::ERROR_OK) {
     return false;
   }
 
@@ -141,13 +144,11 @@ bool SHTCXComponent::read_data_(uint16_t *data, uint8_t len) {
     uint8_t crc = sht_crc(buf[j], buf[j + 1]);
     if (crc != buf[j + 2]) {
       ESP_LOGE(TAG, "CRC8 Checksum invalid! 0x%02X != 0x%02X", buf[j + 2], crc);
-      delete[](buf);
       return false;
     }
     data[i] = (buf[j] << 8) | buf[j + 1];
   }
 
-  delete[](buf);
   return true;
 }
 
